-- /makedoc title Documentation for SOOP API Library v1.0
-- /makedoc insert soop_class.e
-- /makedoc insert soop_object.e
-- /makedoc insert soop_debug.e
-- /makedoc insert soop_ll.e
-- /makedoc insert soop_vars.e
-- /makedoc insert soop_constants.e
-- /topic introduction
-- /code
-- Simplistic Object Oriented Programming (SOOP) API Library
-- Version 1.0 Beta
-- Created by: Mario Steele <eumario@trilake.net>
-- Website: http://enchantedblade.trilake.net/
--
-- Filename: soop.e
-- Details: This includes the entire soop system for use in the language.
--
-- SOOP is a Object Oriented Programming library, that is a basic re-write
-- of the entire PHPized Object Oriented Library.  This library still uses
-- the basics of what POOP has entailed, with function such as Class(), new(),
-- method_func(), and so on.  It's been renamed, and re-organized, to be easier
-- to maintain, as well as readability to the end user.
--
-- SOOP is split up into several files, which deal with each section of the
-- SOOP Architecture.  When you want to use SOOP, you simply include
-- soop.e into your program, and everything will be ready for you.
--
-- SOOP is free to re-distribute, modify, or hack in general to your liking.
-- However, any redistributions of the SOOP library require that you clearly
-- mark in the library what section of code you modified, and a modified copy
-- of the source code be re-submitted to the author of the library.
-- /endcode
-- It is suggested that you look at /"Documentation Notes" before continuing
-- with reading this documentation.
--
-- For Release Notes, see relnotes.txt

-- /topic Documentation Notes
-- /info
--  General Notes about how the Documentation is written.
-- Some terms used in Euphoria, and the SOOP library can be confusing, as some have
-- similar names, and therefore may confuse most.  This will help assist thoes
-- trying to learn how to use the library, via the Documentation.
--
-- /li eInt - Refers to the Euphoria Integer Type
-- /li eAtom - Refers to the Euphoria Atom Type
-- /li eSeq - Refers to the Euphoria Sequence Type
-- /li eObj - Refers to the Euphoria Object Type
-- /li eConst - Refers to a Euphoria Constant
-- /li sfStr -- Refers to a Euphoria Sequence, that is a String Encoded, flat sequence.  EG: "This is a String"
-- /li sObj - Refers to the SOOP Object Instance Pointer
-- /li dt - Refers to a Data Type
-- /li mfunc - Refers to the SOOP Methods that are Functions
-- /li mproc - Refers to the SOOP Methods that are Procedures
-- /li func - Refers to the Euphoria function
-- /li proc - Refers to the Euphoria procedure

-- /topic SOOP Low Level API
-- /info
--  A general warning about Low Level API in SOOP
-- Okay, since the re-design of the library has changed, I've come to the decision to open
-- up SOME of the low level routines for handling SOOP things.  However, I will make several
-- notes right now.
--
-- Low Level routines should be used at your own discression.  I will not support
-- use of the low level routines, and therefore, you acknowledge that you do it of
-- your own free will.
--
-- Low Level Routines are mainly for retrival of data from the SOOP Core.  It's not meant
-- to expose anything vital, should you patch the SOOP core, you do it at your own discression.
--
-- I will help in understand things, but I'm simply stating that attempting to modify
-- how this library works, is not for the light of heart, and should have a good understanding
-- of how the library works.
--
-- With that being said, have fun looking through the Low Level Routines!

include binary.e			-- Jordah Ferguson's Binary.e Library for serialization of
							-- Euphoria Objects
include soop_ll.e			-- SOOP's low-level functions (Euphoria Builtins as well)
include soop_constants.e	-- Basic constants that are used throughout the program
include soop_vars.e			-- DataType system for SOOP
include soop_class.e		-- Class Definition system
include soop_object.e		-- Class Object system
include soop_debug.e		-- Debugging system