-- Simplistic Object Oriented Programming (SOOP) API Library
-- Version 1.0 Beta
-- Created by: Mario Steele <eumario@trilake.net>
-- Website: http://enchantedblade.trilake.net/
--
-- Filename: soop_constants.e
-- Details: This library holds all of the constants used by the other libraries

-- /topic SOOP Constants
-- /info
--  Constants used throughout the SOOP library
-- These constants are used throughout the library, for accessing common values
-- in much the same fashion as Win32lib uses constants such as Window, EditText,
-- and other similar ones.

-- /topic SOOP Constants
-- /const this
-- /desc Refers to the current object in question in methods.
--
-- The 'this' constant is used to determin which object is currently being
-- used for such things as method_proc(), method_func(), method_id()'s with
-- call_func()/call_proc(), and External Library Call-backs with method_call_back().
-- It's primary usage, is for calling other methods on the same object, or storing data on the
-- current object in question.  The 'this' variable is only valid within methods
-- and cannot be used outside of a Object Scope.  (Like with Normal Euphoria
-- procedures, functions not explictly defined as a method, or in top level code.)
global constant
	this = -#FFFFFF,

-- /topic SOOP Constants
-- /const DBG_CLASS
-- /desc Tells the debug class to output debug information from the Class enviroment.
--
-- DBG_CLASS is expressly used in the Debug Class of the SOOP library, and is
-- meant as a flag to with_debug().  It is used to determin which data should
-- be outputted to the debug file, when a call to the method write is done.
-- This flag can be combined with DBG_OBJECT via DBG_CLASS+DBG_OBJECT to tell
-- the method write to write both Class and Object data out to the file.
	DBG_CLASS = 1,
-- /topic SOOP Constants
-- /const DBG_OBJECT
-- /desc Tells the debug class to output debug information from the Object enviroment.
--
-- DBG_OBJECT is expressly used in the Debug Class of the SOOP library, and is
-- meant as a flag to with_debug().  It is used to determin which data should
-- be outputted to the debug file, when a call to the method write is done.
-- This flag can be combined with DBG_CLASS via DBG_CLASS+DBG_OBJECT to tell
-- the method write to write both Class and Object data out to the file.
	DBG_OBJECT = 2,

	-- General Errors
	ERR_STRING = soop_adderror("A string identifier is required."),
	
	-- Class Definition Errors
	ERR_INDEF = soop_adderror("Already in a class definition."),
	ERR_NINDEF = soop_adderror("Not in a class definition."),
	ERR_CLASSDEF = soop_adderror("Class has already been defined."),
	ERR_MEMBERDEF = soop_adderror("Member %s is already defined in class %s."),
	ERR_METHODDEF = soop_adderror("Method %s is already defined in class %s."),
	ERR_AINHERIT = soop_adderror("Class %s already inherits from class %s."),
	ERR_CNDEF = soop_adderror("Class %s has not been defined yet."),
	
	-- Datatype Definition Errors
	ERR_DT_INDEF = soop_adderror("Already in a data type definition."),
	ERR_DT_NINDEF = soop_adderror("Not in a data type definition."),
	ERR_DT_INVTYPE = soop_adderror("Invalid type given."),
	
	-- Object Core Errors
	ERR_OBJ_ARGLENGTH = soop_adderror("Argument length do not equal to refrences."),
	ERR_OBJ_NOCLASS = soop_adderror("Class not defined at this point."),
	ERR_OBJ_INVOBJ = soop_adderror("Invalid object pointer given."),
	ERR_OBJ_MEM404 = soop_adderror("Member %s was not found on object instance %d."),
	ERR_OBJ_DTNOPOINT = soop_adderror("Member type does not allow pointer retrival!"),
	ERR_OBJ_DTNOPOINT2 = soop_adderror("Member type does not allow pointer assignment."),
	ERR_OBJ_MET404 = soop_adderror("Method %s was not found on object instance %d.")