-- Simplistic Object Oriented Programming (SOOP) API Library
-- Version 1.0 Beta
-- Created by: Mario Steele <eumario@trilake.net>
-- Website: http://enchantedblade.trilake.net/
--
-- Filename: soop_debug.e
-- Details: This is a Class instance that allows for
-- running debug information on Classes, and Objects

-- /topic SOOP Debug
-- /info
--  A Class built on SOOP to display Debug Information
--
-- SOOP Debug is a Class to debug the SOOP Enviroment for the programmer.
-- This provides vital information for them to see, when they need it.

-- /topic SOOP Debug
-- /func sObject()
-- /ret String of Text
-- /desc This returns a Printable String of the SOOP Enviroment for debugging purposes.
--
-- This function returns an ASCII character string to display the current status of the
-- SOOP object instance enviroment.  It is actually a Method, not a euphoria function,
-- and therefore requires it's utilization through the method_func() routine.
-- It can be accessed through the DEBUG Object after using with_debug() to initalize it.
	function Debug_sObject()
		sequence Members, string, members
		integer validObj
		string = sprintf("Object Debug:\n%s\n",{repeat('-',40)})
		string &= sprintf("Number of Objects in Memory: %d\n",{soop_OBJ_Num()})
		members = ""
		validObj = 0
		for x = 1 to soop_OBJ_Num() do
			if ValidObject(x) then
				validObj += 1
				members &= sprintf("Object Instance %d:\nClass: %s\n%s\n\n",{x,ClassType(x),repeat('-',40)})
				Members = soop_OBJ_EnumMembers(x)
				for y = 1 to length(Members) do
					members &= sprintf("Member %s: ",{Members[y]})
					if DataType_AllowPointer(soop_MemberType(soop_OBJ_ClassIX(x),y)) then
						if integer(soop_OBJ_MemberVal(x,y)) then
							members &= sprintf("[PTR:%d] ",{soop_OBJ_MemberVal(x,y)})
						elsif atom(soop_OBJ_MemberVal(x,y)) then
							members &= sprintf("[PTR:#%x] ",{soop_OBJ_MemberVal(x,y)})
						else
							members &= sprintf("[PTR:%s] ",{soop_sprints(soop_OBJ_MemberVal(x,y))})
						end if
					end if
					members &= soop_sprints(DataType_Retrieve(soop_MemberType(soop_OBJ_ClassIX(x),y),soop_OBJ_MemberVal(x,y)))
					members &= "\n"
				end for
			end if
		end for
		string &= sprintf("Number of valid objects in Memory: %d\n",{validObj})
		string &= sprintf("\nObject instance Member Data:\n%s\n%s",{repeat('-',40),members})
		return string
	end function

-- /topic SOOP Debug
-- /func sClass()
-- /ret String of Text
-- /desc This returns a Printable String of the SOOP Enviroment for debugging purposes.
--
-- This function returns an ASCII character string to display the current status of the
-- SOOP class definition enviroment.  It is actually a Method, not a euphoria function,
-- and therefore requires it's utilization through the method_func() routine.
-- It can be accessed through the DEBUG Object after using with_debug() to initalize it.	
	function Debug_sClass()
		sequence Classes, string, Members
		string = sprintf("Class Debug:\n%s\n",{repeat('-',40)})
		Classes = soop_EnumClasses()
		string &= sprintf("Number of Classes defined: %d\n\n",length(Classes))
		for x = 1 to length(Classes) do
			string &= sprintf("Class: %s\n%s\n",{Classes[x],repeat('-',40)})
			Members = soop_EnumMembers(x)
			string &= sprintf("Number of Members: %d\n",{length(Members)})
			for y = 1 to length(Members) do
				string &= sprintf("Member Name: %s\n       Type: %s\n",{Members[y],soop_MemberTypeName(x,y)})
			end for
			Members = soop_EnumMethods(x)
			string &= sprintf("\nNumber of Methods: %d\n",{length(Members)})
			for y = 1 to length(Members) do
				string &= sprintf("Method: %s()\n",{Members[y]})
			end for
			string &= "\n"
		end for
		return string
	end function

-- /topic SOOP Debug
-- /proc Object()
-- /desc This writes a String to the file defined in the SOOP Debug Class Object.
--
-- This proceudre writes an ASCII Character string to the file given in the SOOP Debug
-- object Class. It is actually a method, not a euphoria procedure, and therefore
-- reqires it's utilization through methid_proc() routine.
-- It can only be accessed through the DEBUG Object after using with_debug() to initalize it.
	procedure Debug_Object()
		puts(oget(this,"fh"),Debug_sObject())
	end procedure

-- /topic SOOP Debug
-- /proc Class()
-- /desc This writes a String to the file defined in the SOOP Debug Class Object.
--
-- This proceudre writes an ASCII Character string to the file given in the SOOP Debug
-- object. It is actually a method, not a euphoria procedure, and therefore
-- reqires it's utilization through methid_proc() routine.
-- It can only be accessed through the DEBUG Object after using with_debug() to initalize it.
	procedure Debug_Class()
		puts(oget(this,"fh"),Debug_sClass())
	end procedure

-- /topic SOOP Debug
-- /proc write()
-- /desc This writes the enviroment based on the flags given in the SOOP Debug Class Object.
--
-- This procedure writes an ASCII Character string tothe file give in the SOOP Debug
-- object, based apon the flags stored in the object.  You can change what information
-- is written to file by utilizing oset(DEBUG,"flag",DBG_CLASS), oset(DEBUG,"flag",DBG_OBJECT)
-- or oset(DEBUG,"flag",DBG_CLASS+OBJECT).  This is not a normal Euphoria function, and
-- therefore requires the usage of method_proc() to utitilize it.
	procedure Debug_write()
		if oget(this,"fh") = -1 then
			oset(this,"fh",open(oget(this,"file"),"wb"))
			if oget(this,"fh") = -1 then
				soop_error("[DEBUG] write",{},{"Unable to write debug information to file: %s",oget(this,"file")})
			end if
		end if
		if oget(this,"flag") then end if
		if olast(this) = 1 then
			Debug_Class()
		elsif olast(this) = 2 then
			Debug_Object()
		elsif olast(this) = 3 then
			Debug_Class()
			Debug_Object()
		end if
	end procedure

global atom DEBUG
DEBUG = 0

-- /topic SOOP Debug
-- /proc with_debug( eInt flags )
-- /desc Initalizes the DEBUG Class, and stores the Default Values
--
-- with_debug() is a way to show how to dynamically create classes,
-- and define routines outside of it, so that new classes can be
-- derived from existing functions.  with_debug() does just that,
-- initalizes the Debug Class definition, and creates the
-- object instance, storing it in DEBUG.  This allows for the DEBUG
-- class to exist, but not utilized unless with_debug() is used.
-- This keeps the Class Enviroment, and the Object Enviroment clean
-- for your own Class and Objects.
global procedure with_debug(integer flags)
	sequence temp
	Class("SOOP_Debug")
		Member("fh",INTEGER)
		Member("flag",INTEGER)
		Member("file",STRING)
		Member("progDir",STRING)
		Method("sObject",routine_id("Debug_sObject"))
		Method("sClass",routine_id("Debug_sClass"))
		Method("Object",routine_id("Debug_Object"))
		Method("Class",routine_id("Debug_Class"))
		Method("write",routine_id("Debug_write"))
	EndClass()
	DEBUG = new("SOOP_Debug",{})
	oset(DEBUG,{"fh","flag","file"},{-1,flags,"soop.err"})
	temp = command_line()
	if find('\\',temp[2]) then
		temp = soop_reverse(temp[2])
		temp = soop_reverse(temp[find('\\',temp)..length(temp)])
	else
		temp = machine_func(23,0) -- current_dir()
	end if
	oset(DEBUG,"progDir",temp)
end procedure

-- /topic SOOP Debug
-- /proc debug_file( sfStr )
-- /desc Stores a filename for use with the Debug Class.
--
-- This is used to store the file name or path and file name
-- to store debug information in, when the method write() is called.
-- This is similar to Euphoria's crash_file() routine, defined in
-- machine.e.
global procedure debug_file(sequence file)
	if ValidObject(DEBUG) then
		oset(DEBUG,"file",file)
	else
		soop_error("debug_file",{file},"with_debug() hasn't been executed yet!")
	end if
end procedure

function SOOP_Crash(object dat)
	integer fh
	if oget(DEBUG,"fh") = -1 then
		if match(":\\",oget(DEBUG,"file") then
			oset(DEBUG,"fh",open(oget(DEBUG,"file"),"wb"))
		else
			oset(DEBUG,"fh",open(oget(DEBUG,"progDir") & oget(DEBUG,"file"),"wb"))
		end if
		if oget(DEBUG,"fh") = -1 then
			soop_error("[DEBUG] with_crash_routine",{},{"Unable to open file %s for writting.",oget(DEBUG,"file")})
		end if
	end if
	method_proc(DEBUG,"write",{})
	if oget(DEBUG,"fh") then end if
	if equal({},OurError) then
		puts(olast(DEBUG),"\n\nEuphoria Error:\n")
		fh = open("ex.err","rb")
		if fh = -1 then
			puts(olast(DEBUG),"Unable to get information from ex.err\n")
		else
			for x = 1 to 3 do
				puts(olast(DEBUG),gets(fh))
			end for
			close(fh)
		end if
	else
		puts(olast(DEBUG),"\n\nSOOP Library Error:\n" & OurError & "\n")
	end if
	close(olast(DEBUG))
	return 0
end function

-- /topic SOOP Debug
-- /proc with_crash_routine()
-- /desc Enables Crash Routine Support (2.5+)
--
-- with_crash_routine() allows for Programmers using Euphoria 2.5+ to utilize the
-- crash_routine() system, to write out debug information to disk.  This function
-- is not comptable on Euphoria Interpreter's Pre 2.5 Release.  It allows for debug
-- information to be written, weither the SOOP Library crashes out, or Normal Euphoria
-- crashes out with a normal Run-Time Error, or Machine Exception error.  It allows for
-- the SOOP Enviroment to be available to the End User, in a Readable Format, so that
-- they can better figure out if the problem occured within the SOOP Lirbary, or
-- somewhere else in their own program.  The SOOP Crash Routine is also smart enough
-- to figure out if it was a SOOP Error that occured, or if Euphoria errored out, and
-- write the first 3 lines of the ex.err to soop.err (Default Crash File for SOOP).
-- This will not be compatable if crash_file() has been used, since it looks for
-- ex.err.
global procedure with_crash_routine()
	if not ValidObject(DEBUG) then
		with_debug(DBG_CLASS+DBG_OBJECT)
	end if
	machine_proc(66,routine_id("SOOP_Crash"))
end procedure